from importlib import import_module

try:
    from django.core.urlresolvers import reverse
except ImportError:  # Django 1.11
    from django.urls import reverse

from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _

from jet.dashboard import modules
from jet.dashboard.models import UserDashboardModule
from jet.ordered_set import OrderedSet
from jet.utils import context_to_dict, get_admin_site_name

try:
    from django.template.context_processors import csrf
except ImportError:
    from django.core.context_processors import csrf


class Dashboard(object):
    """
    Base dashboard class. All custom dashboards should inherit it.
    """

    #: Number of columns in which widgets can be placed
    columns = 2

    #: Dashboard Modules (widgets) that dashboard is filled with, when the user open it for the first time
    #:
    #: List of dashboard module **instances**
    children = None

    #: Dashboard Modules (widgets) that user can add to dashboard at any time
    # (not created when the user open dashboard for the first time)
    #:
    #: List of dashboard module **classes**
    available_children = None
    app_label = None
    context = None
    modules = None

    class Media:
        css = ()
        js = ()

    def __init__(self, context, **kwargs):
        for key in kwargs:
            if hasattr(self.__class__, key):
                setattr(self, key, kwargs[key])
        self.children = self.children or []
        self.available_children = self.available_children or []
        self.set_context(context)

    def set_context(self, context):
        self.context = context
        self.init_with_context(context)
        self.load_modules()

    def init_with_context(self, context):
        """
        Override this method to fill your custom **Dashboard** class with widgets.
        You should add your widgets to ``children`` and ``available_children`` attributes.

        Usage example:

        .. code-block:: python

            from django.utils.translation import ugettext_lazy as _
            from jet.dashboard import modules
            from jet.dashboard.dashboard import Dashboard, AppIndexDashboard


            class CustomIndexDashboard(Dashboard):
                columns = 3

                def init_with_context(self, context):
                    self.available_children.append(modules.LinkList)
                    self.children.append(modules.LinkList(
                        _('Support'),
                        children=[
                            {
                                'title': _('Django documentation'),
                                'url': 'http://docs.djangoproject.com/',
                                'external': True,
                            },
                            {
                                'title': _('Django "django-users" mailing list'),
                                'url': 'http://groups.google.com/group/django-users',
                                'external': True,
                            },
                            {
                                'title': _('Django irc channel'),
                                'url': 'irc://irc.freenode.net/django',
                                'external': True,
                            },
                        ],
                        column=0,
                        order=0
                    ))

        """
        pass

    def load_module(self, module_fullname):
        package, module_name = module_fullname.rsplit(".", 1)
        package = import_module(package)
        module = getattr(package, module_name)

        return module

    def create_initial_module_models(self, user):
        module_models = []

        i = 0

        for module in self.children:
            column = module.column if module.column is not None else i % self.columns
            order = module.order if module.order is not None else int(i / self.columns)

            module_models.append(
                UserDashboardModule.objects.create(
                    title=module.title,
                    app_label=self.app_label,
                    user=user.pk,
                    module=module.fullname(),
                    column=column,
                    order=order,
                    settings=module.dump_settings(),
                    children=module.dump_children(),
                )
            )
            i += 1

        return module_models

    def load_modules(self):
        module_models = UserDashboardModule.objects.filter(
            app_label=self.app_label, user=self.context["request"].user.pk
        ).all()

        if len(module_models) == 0:
            module_models = self.create_initial_module_models(
                self.context["request"].user
            )

        loaded_modules = []

        for module_model in module_models:
            module_cls = module_model.load_module()
            if module_cls is not None:
                module = module_cls(model=module_model, context=self.context)
                loaded_modules.append(module)

        self.modules = loaded_modules

    def render(self):
        context = context_to_dict(self.context)
        context.update(
            {
                "columns": range(self.columns),
                "modules": self.modules,
                "app_label": self.app_label,
            }
        )
        context.update(csrf(context["request"]))

        return render_to_string("jet.dashboard/dashboard.html", context)

    def render_tools(self):
        context = context_to_dict(self.context)
        context.update(
            {
                "children": self.children,
                "app_label": self.app_label,
                "available_children": self.available_children,
            }
        )
        context.update(csrf(context["request"]))

        return render_to_string("jet.dashboard/dashboard_tools.html", context)

    def media(self):
        unique_css = OrderedSet()
        unique_js = OrderedSet()

        for js in getattr(self.Media, "js", ()):
            unique_js.add(js)
        for css in getattr(self.Media, "css", ()):
            unique_css.add(css)

        for module in self.modules:
            for js in getattr(module.Media, "js", ()):
                unique_js.add(js)
            for css in getattr(module.Media, "css", ()):
                unique_css.add(css)

        class Media:
            css = list(unique_css)
            js = list(unique_js)

        return Media


class AppIndexDashboard(Dashboard):
    def get_app_content_types(self):
        return (self.app_label + ".*",)

    def models(self):
        return (self.app_label + ".*",)


class DefaultIndexDashboard(Dashboard):
    columns = 3

    def init_with_context(self, context):
        self.available_children.append(modules.LinkList)
        self.available_children.append(modules.Feed)

        site_name = get_admin_site_name(context)
        # append a link list module for "quick links"
        self.children.append(
            modules.LinkList(
                _("Quick links"),
                layout="inline",
                draggable=False,
                deletable=False,
                collapsible=False,
                children=[
                    [_("Return to site"), "/"],
                    [_("Change password"), reverse("%s:password_change" % site_name)],
                    [_("Log out"), reverse("%s:logout" % site_name)],
                ],
                column=0,
                order=0,
            )
        )

        # append an app list module for "Applications"
        self.children.append(
            modules.AppList(_("Applications"), exclude=("auth.*",), column=1, order=0)
        )

        # append an app list module for "Administration"
        self.children.append(
            modules.AppList(_("Administration"), models=("auth.*",), column=2, order=0)
        )

        # append a recent actions module
        self.children.append(
            modules.RecentActions(_("Recent Actions"), 10, column=0, order=1)
        )

        # append a feed module
        self.children.append(
            modules.Feed(
                _("Latest Django News"),
                feed_url="http://www.djangoproject.com/rss/weblog/",
                limit=5,
                column=1,
                order=1,
            )
        )

        # append another link list module for "support".
        self.children.append(
            modules.LinkList(
                _("Support"),
                children=[
                    {
                        "title": _("Django documentation"),
                        "url": "http://docs.djangoproject.com/",
                        "external": True,
                    },
                    {
                        "title": _('Django "django-users" mailing list'),
                        "url": "http://groups.google.com/group/django-users",
                        "external": True,
                    },
                    {
                        "title": _("Django irc channel"),
                        "url": "irc://irc.freenode.net/django",
                        "external": True,
                    },
                ],
                column=2,
                order=1,
            )
        )


class DefaultAppIndexDashboard(AppIndexDashboard):
    def init_with_context(self, context):
        self.available_children.append(modules.LinkList)

        self.children.append(
            modules.ModelList(
                title=_("Application models"), models=self.models(), column=0, order=0
            )
        )
        self.children.append(
            modules.RecentActions(
                include_list=self.get_app_content_types(), column=1, order=0
            )
        )


class DashboardUrls(object):
    _urls = []

    def get_urls(self):
        return self._urls

    def register_url(self, url):
        self._urls.append(url)

    def register_urls(self, urls):
        self._urls.extend(urls)


urls = DashboardUrls()



# encoding: utf-8
import datetime
import json

from django import forms

try:
    from django.core.urlresolvers import reverse
except ImportError:  # Django 1.11
    from django.urls import reverse

from django.conf import settings
from django.forms import Widget
from django.utils import formats
from django.utils.encoding import force_text
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.text import capfirst
from django.utils.translation import ugettext_lazy as _

from jet.dashboard.modules import DashboardModule

try:
    from urllib import request
    from urllib.error import HTTPError, URLError
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode

    import urllib2 as request
    from urllib2 import HTTPError, URLError

JET_MODULE_YANDEX_METRIKA_CLIENT_ID = getattr(
    settings, "JET_MODULE_YANDEX_METRIKA_CLIENT_ID", ""
)
JET_MODULE_YANDEX_METRIKA_CLIENT_SECRET = getattr(
    settings, "JET_MODULE_YANDEX_METRIKA_CLIENT_SECRET", ""
)


class YandexMetrikaClient:
    OAUTH_BASE_URL = "https://oauth.yandex.ru/"
    API_BASE_URL = "https://api-metrika.yandex.ru/"
    CLIENT_ID = JET_MODULE_YANDEX_METRIKA_CLIENT_ID
    CLIENT_SECRET = JET_MODULE_YANDEX_METRIKA_CLIENT_SECRET

    def __init__(self, access_token=None):
        self.access_token = access_token

    def request(self, base_url, url, data=None, headers=None):
        url = "%s%s" % (base_url, url)

        if data is not None:
            data = urlencode(data).encode()

        if headers is None:
            headers = {}

        req = request.Request(url, data, headers)

        try:
            f = request.urlopen(req)
            result = f.read().decode("utf8")
            result = json.loads(result)
        except URLError as e:
            return None, e

        return result, None

    def get_oauth_authorize_url(self, state=""):
        return (
            "%sauthorize"
            "?response_type=code"
            "&state=%s"
            "&client_id=%s" % (self.OAUTH_BASE_URL, state, self.CLIENT_ID)
        )

    def oauth_request(self, url, data=None):
        return self.request(self.OAUTH_BASE_URL, url, data)

    def oath_token_request(self, code):
        data = {
            "grant_type": "authorization_code",
            "code": code,
            "client_id": self.CLIENT_ID,
            "client_secret": self.CLIENT_SECRET,
        }
        return self.oauth_request("token", data)

    def api_request(self, url, data=None):
        headers = None
        if self.access_token is not None:
            headers = {"Authorization": "OAuth %s" % self.access_token}
        return self.request(self.API_BASE_URL, url, data, headers)

    def api_counters_request(self):
        return self.api_request("counters.json")

    def api_stat_traffic_summary(self, counter, date1, date2, group=None):
        if group is None:
            group = "day"
        return self.api_request(
            "stat/traffic/summary.json?id=%s&date1=%s&date2=%s&group=%s"
            % (counter, date1.strftime("%Y%m%d"), date2.strftime("%Y%m%d"), group)
        )


class AccessTokenWidget(Widget):
    module = None

    def render(self, name, value, attrs=None):
        if value and len(value) > 0:
            link = '<a href="%s">%s</a>' % (
                reverse(
                    "jet-dashboard:yandex-metrika-revoke",
                    kwargs={"pk": self.module.model.pk},
                ),
                force_text(_("Revoke access")),
            )
        else:
            link = '<a href="%s">%s</a>' % (
                reverse(
                    "jet-dashboard:yandex-metrika-grant",
                    kwargs={"pk": self.module.model.pk},
                ),
                force_text(_("Grant access")),
            )

        if value is None:
            value = ""

        return format_html(
            '%s<input type="hidden" name="access_token" value="%s">' % (link, value)
        )


class YandexMetrikaSettingsForm(forms.Form):
    access_token = forms.CharField(label=_("Access"), widget=AccessTokenWidget)
    counter = forms.ChoiceField(label=_("Counter"))
    period = forms.ChoiceField(
        label=_("Statistics period"),
        choices=(
            (0, _("Today")),
            (6, _("Last week")),
            (30, _("Last month")),
            (31 * 3 - 1, _("Last quarter")),
            (364, _("Last year")),
        ),
    )

    def set_module(self, module):
        self.fields["access_token"].widget.module = module
        self.set_counter_choices(module)

    def set_counter_choices(self, module):
        counters = module.counters()
        if counters is not None:
            self.fields["counter"].choices = (("", "-- %s --" % force_text(_("none"))),)
            self.fields["counter"].choices.extend(
                map(lambda x: (x["id"], x["site"]), counters)
            )
        else:
            label = (
                force_text(_("grant access first"))
                if module.access_token is None
                else force_text(_("counters loading failed"))
            )
            self.fields["counter"].choices = (("", "-- %s -- " % label),)


class YandexMetrikaChartSettingsForm(YandexMetrikaSettingsForm):
    show = forms.ChoiceField(
        label=_("Show"),
        choices=(
            ("visitors", capfirst(_("visitors"))),
            ("visits", capfirst(_("visits"))),
            ("page_views", capfirst(_("views"))),
        ),
    )
    group = forms.ChoiceField(
        label=_("Group"),
        choices=(
            ("day", _("By day")),
            ("week", _("By week")),
            ("month", _("By month")),
        ),
    )


class YandexMetrikaPeriodVisitorsSettingsForm(YandexMetrikaSettingsForm):
    group = forms.ChoiceField(
        label=_("Group"),
        choices=(
            ("day", _("By day")),
            ("week", _("By week")),
            ("month", _("By month")),
        ),
    )


class YandexMetrikaBase(DashboardModule):
    settings_form = YandexMetrikaSettingsForm
    ajax_load = True
    contrast = True
    period = None
    access_token = None
    expires_in = None
    token_type = None
    counter = None
    error = None

    def settings_dict(self):
        return {
            "period": self.period,
            "access_token": self.access_token,
            "expires_in": self.expires_in,
            "token_type": self.token_type,
            "counter": self.counter,
        }

    def load_settings(self, settings):
        try:
            self.period = int(settings.get("period"))
        except TypeError:
            self.period = 0
        self.access_token = settings.get("access_token")
        self.expires_in = settings.get("expires_in")
        self.token_type = settings.get("token_type")
        self.counter = settings.get("counter")

    def init_with_context(self, context):
        raise NotImplementedError(
            "subclasses of YandexMetrika must provide a init_with_context() method"
        )

    def counters(self):
        client = YandexMetrikaClient(self.access_token)
        counters, exception = client.api_counters_request()

        if counters is not None:
            return counters["counters"]
        else:
            return None

    def format_grouped_date(self, date, group):
        if group == "week":
            date = u"%s — %s" % (
                (date - datetime.timedelta(days=7)).strftime("%d.%m"),
                date.strftime("%d.%m"),
            )
        elif group == "month":
            date = date.strftime("%b, %Y")
        else:
            date = formats.date_format(date, "DATE_FORMAT")
        return date

    def counter_attached(self):
        if self.access_token is None:
            self.error = mark_safe(
                _(
                    'Please <a href="%s">attach Yandex account and choose Yandex Metrika counter</a> to start using widget'
                )
                % reverse("jet-dashboard:update_module", kwargs={"pk": self.model.pk})
            )
            return False
        elif self.counter is None:
            self.error = mark_safe(
                _(
                    'Please <a href="%s">select Yandex Metrika counter</a> to start using widget'
                )
                % reverse("jet-dashboard:update_module", kwargs={"pk": self.model.pk})
            )
            return False
        else:
            return True

    def api_stat_traffic_summary(self, group=None):
        if self.counter_attached():
            date1 = datetime.datetime.now() - datetime.timedelta(days=self.period)
            date2 = datetime.datetime.now()

            client = YandexMetrikaClient(self.access_token)
            result, exception = client.api_stat_traffic_summary(
                self.counter, date1, date2, group
            )

            if exception is not None:
                error = _("API request failed.")
                if isinstance(exception, HTTPError) and exception.code == 403:
                    error += _(
                        ' Try to <a href="%s">revoke and grant access</a> again'
                    ) % reverse(
                        "jet-dashboard:update_module", kwargs={"pk": self.model.pk}
                    )
                self.error = mark_safe(error)
            else:
                return result


class YandexMetrikaVisitorsTotals(YandexMetrikaBase):
    """
    Yandex Metrika widget that shows total number of visitors, visits and viewers for a particular period of time.
    Period may be following: Today, Last week, Last month, Last quarter, Last year
    """

    title = _("Yandex Metrika visitors totals")
    template = "jet.dashboard/modules/yandex_metrika_visitors_totals.html"

    #: Which period should be displayed. Allowed values - integer of days
    period = None

    def __init__(self, title=None, period=None, **kwargs):
        kwargs.update({"period": period})
        super(YandexMetrikaVisitorsTotals, self).__init__(title, **kwargs)

    def init_with_context(self, context):
        result = self.api_stat_traffic_summary()

        if result is not None:
            try:
                self.children.append(
                    {"title": _("visitors"), "value": result["totals"]["visitors"]}
                )
                self.children.append(
                    {"title": _("visits"), "value": result["totals"]["visits"]}
                )
                self.children.append(
                    {"title": _("views"), "value": result["totals"]["page_views"]}
                )
            except KeyError:
                self.error = _("Bad server response")


class YandexMetrikaVisitorsChart(YandexMetrikaBase):
    """
    Yandex Metrika widget that shows visitors/visits/viewer chart for a particular period of time.
    Data is grouped by day, week or month
    Period may be following: Today, Last week, Last month, Last quarter, Last year
    """

    title = _("Yandex Metrika visitors chart")
    template = "jet.dashboard/modules/yandex_metrika_visitors_chart.html"
    style = "overflow-x: auto;"

    #: Which period should be displayed. Allowed values - integer of days
    period = None

    #: What data should be shown. Possible values: ``visitors``, ``visits``, ``page_views``
    show = None

    #: Sets grouping of data. Possible values: ``day``, ``week``, ``month``
    group = None
    settings_form = YandexMetrikaChartSettingsForm

    class Media:
        js = (
            "jet.dashboard/vendor/chart.js/Chart.min.js",
            "jet.dashboard/dashboard_modules/yandex_metrika.js",
        )

    def __init__(self, title=None, period=None, show=None, group=None, **kwargs):
        kwargs.update({"period": period, "show": show, "group": group})
        super(YandexMetrikaVisitorsChart, self).__init__(title, **kwargs)

    def settings_dict(self):
        settings = super(YandexMetrikaVisitorsChart, self).settings_dict()
        settings["show"] = self.show
        settings["group"] = self.group
        return settings

    def load_settings(self, settings):
        super(YandexMetrikaVisitorsChart, self).load_settings(settings)
        self.show = settings.get("show")
        self.group = settings.get("group")

    def init_with_context(self, context):
        result = self.api_stat_traffic_summary(self.group)

        if result is not None:
            try:
                for data in result["data"]:
                    date = datetime.datetime.strptime(data["date"], "%Y%m%d")
                    key = self.show if self.show is not None else "visitors"
                    self.children.append((date, data[key]))
            except KeyError:
                self.error = _("Bad server response")


class YandexMetrikaPeriodVisitors(YandexMetrikaBase):
    """
    Yandex Metrika widget that shows visitors, visits and viewers for a particular period of time.
    Data is grouped by day, week or month
    Period may be following: Today, Last week, Last month, Last quarter, Last year
    """

    title = _("Yandex Metrika period visitors")
    template = "jet.dashboard/modules/yandex_metrika_period_visitors.html"

    #: Which period should be displayed. Allowed values - integer of days
    period = None

    #: Sets grouping of data. Possible values: ``day``, ``week``, ``month``
    group = None
    contrast = False
    settings_form = YandexMetrikaPeriodVisitorsSettingsForm

    def __init__(self, title=None, period=None, group=None, **kwargs):
        kwargs.update({"period": period, "group": group})
        super(YandexMetrikaPeriodVisitors, self).__init__(title, **kwargs)

    def settings_dict(self):
        settings = super(YandexMetrikaPeriodVisitors, self).settings_dict()
        settings["group"] = self.group
        return settings

    def load_settings(self, settings):
        super(YandexMetrikaPeriodVisitors, self).load_settings(settings)
        self.group = settings.get("group")

    def init_with_context(self, context):
        result = self.api_stat_traffic_summary(self.group)

        if result is not None:
            try:
                for data in reversed(result["data"]):
                    date = datetime.datetime.strptime(data["date"], "%Y%m%d")
                    date = self.format_grouped_date(date, self.group)
                    self.children.append((date, data))
            except KeyError:
                self.error = _("Bad server response")



try:
    from django.core.management.base import NoArgsCommand
except ImportError:
    from django.core.management import BaseCommand as NoArgsCommand

from jet.utils import get_app_list


class Command(NoArgsCommand):
    help = "Generates example of JET custom apps setting"
    item_order = 0

    def handle(self, *args, **options):
        if args:
            raise CommandError("Command doesn't accept any arguments")
        return self.handle_noargs(**options)

    def handle_noargs(self, **options):
        class User:
            is_active = True
            is_staff = True
            is_superuser = True

            def has_module_perms(self, app):
                return True

            def has_perm(self, object):
                return True

        class Request:
            user = User()

        app_list = get_app_list({"request": Request(), "user": None})

        self.stdout.write(
            "# Add this to your settings.py to customize applications and models list"
        )
        self.stdout.write("JET_SIDE_MENU_CUSTOM_APPS = [")

        for app in app_list:
            app_label = app.get("app_label", app.get("name"))

            self.stdout.write("    ('%s', [" % app_label)
            for model in app["models"]:
                self.stdout.write("        '%s'," % model["object_name"])
            self.stdout.write("    ]),")

        self.stdout.write("]")



from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

try:
    from django.utils.encoding import python_2_unicode_compatible
except ImportError:
    from six import python_2_unicode_compatible


@python_2_unicode_compatible
class Bookmark(models.Model):
    url = models.URLField(verbose_name=_("URL"))
    title = models.CharField(verbose_name=_("title"), max_length=255)
    user = models.PositiveIntegerField(verbose_name=_("user"))
    date_add = models.DateTimeField(
        verbose_name=_("date created"), default=timezone.now
    )

    class Meta:
        verbose_name = _("bookmark")
        verbose_name_plural = _("bookmarks")
        ordering = ("date_add",)

    def __str__(self):
        return self.title


@python_2_unicode_compatible
class PinnedApplication(models.Model):
    app_label = models.CharField(verbose_name=_("application name"), max_length=255)
    user = models.PositiveIntegerField(verbose_name=_("user"))
    date_add = models.DateTimeField(
        verbose_name=_("date created"), default=timezone.now
    )

    class Meta:
        verbose_name = _("pinned application")
        verbose_name_plural = _("pinned applications")
        ordering = ("date_add",)

    def __str__(self):
        return self.app_label



from importlib import import_module

from jet.dashboard import settings


def get_current_dashboard(location):
    if location == "index":
        path = settings.JET_INDEX_DASHBOARD
    elif location == "app_index":
        path = settings.JET_APP_INDEX_DASHBOARD
    else:
        raise ValueError("Unknown dashboard location: %s" % location)

    module, cls = path.rsplit(".", 1)

    try:
        module = import_module(module)
        index_dashboard_cls = getattr(module, cls)
        return index_dashboard_cls
    except ImportError:
        return None



from django.views.decorators.http import require_GET, require_POST

from jet.forms import (
    AddBookmarkForm,
    ModelLookupForm,
    RemoveBookmarkForm,
    ToggleApplicationPinForm,
)
from jet.models import Bookmark
from jet.utils import JsonResponse


@require_POST
def add_bookmark_view(request):
    result = {"error": False}
    form = AddBookmarkForm(request, request.POST)

    if form.is_valid():
        bookmark = form.save()
        result.update({"id": bookmark.pk, "title": bookmark.title, "url": bookmark.url})
    else:
        result["error"] = True

    return JsonResponse(result)


@require_POST
def remove_bookmark_view(request):
    result = {"error": False}

    try:
        instance = Bookmark.objects.get(pk=request.POST.get("id"))
        form = RemoveBookmarkForm(request, request.POST, instance=instance)

        if form.is_valid():
            form.save()
        else:
            result["error"] = True
    except Bookmark.DoesNotExist:
        result["error"] = True

    return JsonResponse(result)


@require_POST
def toggle_application_pin_view(request):
    result = {"error": False}
    form = ToggleApplicationPinForm(request, request.POST)

    if form.is_valid():
        pinned = form.save()
        result["pinned"] = pinned
    else:
        result["error"] = True

    return JsonResponse(result)


@require_GET
def model_lookup_view(request):
    result = {"error": False}

    form = ModelLookupForm(request, request.GET)

    if form.is_valid():
        items, total = form.lookup()
        result["items"] = items
        result["total"] = total
    else:
        result["error"] = True

    return JsonResponse(result)



from django.conf import settings

# Dashboard
JET_INDEX_DASHBOARD = getattr(
    settings, "JET_INDEX_DASHBOARD", "jet.dashboard.dashboard.DefaultIndexDashboard"
)
JET_APP_INDEX_DASHBOARD = getattr(
    settings,
    "JET_APP_INDEX_DASHBOARD",
    "jet.dashboard.dashboard.DefaultAppIndexDashboard",
)



from django.conf.urls import url
from django.contrib import messages

try:
    from django.core.urlresolvers import reverse
except ImportError:  # Django 1.11
    from django.urls import reverse

from django.http import HttpResponse
from django.shortcuts import redirect
from django.utils.translation import ugettext_lazy as _

from jet.dashboard import dashboard
from jet.dashboard.dashboard_modules.yandex_metrika import YandexMetrikaClient
from jet.dashboard.models import UserDashboardModule


def yandex_metrika_grant_view(request, pk):
    client = YandexMetrikaClient()
    return redirect(client.get_oauth_authorize_url(pk))


def yandex_metrika_revoke_view(request, pk):
    try:
        module = UserDashboardModule.objects.get(pk=pk)
        module.pop_settings(("access_token", "expires_in", "token_type", "counter"))
        return redirect(
            reverse("jet-dashboard:update_module", kwargs={"pk": module.pk})
        )
    except UserDashboardModule.DoesNotExist:
        return HttpResponse(_("Module not found"))


def yandex_metrika_callback_view(request):
    try:
        state = request.GET["state"]
        code = request.GET["code"]

        module = UserDashboardModule.objects.get(pk=state)

        client = YandexMetrikaClient()
        result, exception = client.oath_token_request(code)

        if result is None:
            messages.error(request, _("API request failed."))
        else:
            module.update_settings(result)

        return redirect(
            reverse("jet-dashboard:update_module", kwargs={"pk": module.pk})
        )
    except KeyError:
        return HttpResponse(_("Bad arguments"))
    except UserDashboardModule.DoesNotExist:
        return HttpResponse(_("Module not found"))


dashboard.urls.register_urls(
    [
        url(
            r"^yandex-metrika/grant/(?P<pk>\d+)/$",
            yandex_metrika_grant_view,
            name="yandex-metrika-grant",
        ),
        url(
            r"^yandex-metrika/revoke/(?P<pk>\d+)/$",
            yandex_metrika_revoke_view,
            name="yandex-metrika-revoke",
        ),
        url(
            r"^yandex-metrika/callback/$",
            yandex_metrika_callback_view,
            name="yandex-metrika-callback",
        ),
    ]
)



from __future__ import unicode_literals

from django import template

from jet.dashboard.utils import get_current_dashboard

register = template.Library()
assignment_tag = (
    register.assignment_tag
    if hasattr(register, "assignment_tag")
    else register.simple_tag
)


@assignment_tag(takes_context=True)
def get_dashboard(context, location):
    dashboard_cls = get_current_dashboard(location)

    app_label = context["request"].resolver_match.kwargs.get("app_label")

    return dashboard_cls(context, app_label=app_label)


@register.filter
def format_change_message(log_entry):
    # Django 1.10+
    if hasattr(log_entry, "get_change_message"):
        return log_entry.get_change_message()
    else:
        return log_entry.change_message



from django.contrib import messages
from django.core.exceptions import ValidationError

try:
    from django.core.urlresolvers import reverse
except ImportError:  # Django 1.11
    from django.urls import reverse

from django.forms.formsets import formset_factory
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.http import require_GET, require_POST
from django.views.generic import UpdateView

from jet.dashboard.forms import (
    AddUserDashboardModuleForm,
    RemoveDashboardModuleForm,
    ResetDashboardForm,
    UpdateDashboardModuleCollapseForm,
    UpdateDashboardModulesForm,
)
from jet.dashboard.models import UserDashboardModule
from jet.utils import (
    JsonResponse,
    SuccessMessageMixin,
    get_app_list,
    user_is_authenticated,
)


class UpdateDashboardModuleView(SuccessMessageMixin, UpdateView):
    model = UserDashboardModule
    fields = ("title",)
    template_name = "jet.dashboard/update_module.html"
    success_message = _("Widget was successfully updated")
    object = None
    module = None

    def has_permission(self, request):
        return request.user.is_active and request.user.is_staff

    def get_success_url(self):
        if self.object.app_label:
            return reverse(
                "admin:app_list", kwargs={"app_label": self.object.app_label}
            )
        else:
            return reverse("admin:index")

    def get_module(self):
        object = (
            self.object
            if getattr(self, "object", None) is not None
            else self.get_object()
        )
        return object.load_module()

    def get_settings_form_kwargs(self):
        kwargs = {"initial": self.module.settings}

        if self.request.method in ("POST", "PUT"):
            kwargs.update(
                {
                    "data": self.request.POST,
                    "files": self.request.FILES,
                }
            )
        return kwargs

    def get_settings_form(self):
        if self.module.settings_form:
            form = self.module.settings_form(**self.get_settings_form_kwargs())
            if hasattr(form, "set_module"):
                form.set_module(self.module)
            return form

    def get_children_formset_kwargs(self):
        kwargs = {
            "initial": self.module.children,
            "prefix": "children",
        }

        if self.request.method in ("POST", "PUT"):
            kwargs.update(
                {
                    "data": self.request.POST,
                    "files": self.request.FILES,
                }
            )
        return kwargs

    def get_children_formset(self):
        if self.module.child_form:
            return formset_factory(self.module.child_form, can_delete=True, extra=1)(
                **self.get_children_formset_kwargs()
            )

    def clean_children_data(self, children):
        children = list(
            filter(
                lambda item: isinstance(item, dict)
                and item
                and item.get("DELETE") is not True,
                children,
            )
        )
        for item in children:
            item.pop("DELETE")
        return children

    def get_current_app(self):
        app_list = get_app_list({"request": self.request})

        for app in app_list:
            if app.get("app_label", app.get("name")) == self.object.app_label:
                return app

    def get_context_data(self, **kwargs):
        data = super(UpdateDashboardModuleView, self).get_context_data(**kwargs)
        data["title"] = _("Change")
        data["module"] = self.module
        data["settings_form"] = self.get_settings_form()
        data["children_formset"] = self.get_children_formset()
        data["child_name"] = (
            self.module.child_name if self.module.child_name else _("Items")
        )
        data["child_name_plural"] = (
            self.module.child_name_plural
            if self.module.child_name_plural
            else _("Items")
        )
        data["app"] = self.get_current_app()
        return data

    def dispatch(self, request, *args, **kwargs):
        if not self.has_permission(request):
            index_path = reverse("admin:index")
            return HttpResponseRedirect(index_path)

        self.object = self.get_object()
        self.module = self.get_module()(model=self.object)
        return super(UpdateDashboardModuleView, self).dispatch(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        settings_form = self.get_settings_form()
        children_formset = self.get_children_formset()

        data = request.POST.copy()

        if settings_form:
            if settings_form.is_valid():
                settings = settings_form.cleaned_data
                data["settings"] = self.module.dump_settings(settings)
            else:
                return self.form_invalid(self.get_form(self.get_form_class()))

        if children_formset:
            if children_formset.is_valid():
                self.module.children = self.clean_children_data(
                    children_formset.cleaned_data
                )
                data["children"] = self.module.dump_children()
            else:
                return self.form_invalid(self.get_form(self.get_form_class()))

        request.POST = data

        return super(UpdateDashboardModuleView, self).post(request, *args, **kwargs)

    def form_valid(self, form):
        if "settings" in form.data:
            form.instance.settings = form.data["settings"]
        if "children" in form.data:
            form.instance.children = form.data["children"]
        return super(UpdateDashboardModuleView, self).form_valid(form)


@require_POST
def update_dashboard_modules_view(request):
    result = {"error": False}
    form = UpdateDashboardModulesForm(request, request.POST)

    if form.is_valid():
        form.save()
    else:
        result["error"] = True

    return JsonResponse(result)


@require_POST
def add_user_dashboard_module_view(request):
    result = {"error": False}
    form = AddUserDashboardModuleForm(request, request.POST)

    if form.is_valid():
        module = form.save()
        result["id"] = module.pk
        messages.success(request, _("Widget has been successfully added"))

        if module.app_label:
            result["success_url"] = reverse(
                "admin:app_list", kwargs={"app_label": module.app_label}
            )
        else:
            result["success_url"] = reverse("admin:index")
    else:
        result["error"] = True

    return JsonResponse(result)


@require_POST
def update_dashboard_module_collapse_view(request):
    result = {"error": False}

    try:
        instance = UserDashboardModule.objects.get(pk=request.POST.get("id"))
        form = UpdateDashboardModuleCollapseForm(
            request, request.POST, instance=instance
        )

        if form.is_valid():
            module = form.save()
            result["collapsed"] = module.collapsed
        else:
            result["error"] = True
    except UserDashboardModule.DoesNotExist:
        result["error"] = True

    return JsonResponse(result)


@require_POST
def remove_dashboard_module_view(request):
    result = {"error": False}

    try:
        instance = UserDashboardModule.objects.get(pk=request.POST.get("id"))
        form = RemoveDashboardModuleForm(request, request.POST, instance=instance)

        if form.is_valid():
            form.save()
        else:
            result["error"] = True
    except UserDashboardModule.DoesNotExist:
        result["error"] = True

    return JsonResponse(result)


@require_GET
def load_dashboard_module_view(request, pk):
    result = {"error": False}

    try:
        if not user_is_authenticated(request.user) or not request.user.is_staff:
            raise ValidationError("error")

        instance = UserDashboardModule.objects.get(pk=pk, user=request.user.pk)
        module_cls = instance.load_module()
        module = module_cls(model=instance, context={"request": request})
        result["html"] = module.render()
    except (ValidationError, UserDashboardModule.DoesNotExist):
        result["error"] = True

    return JsonResponse(result)


@require_POST
def reset_dashboard_view(request):
    result = {"error": False}
    form = ResetDashboardForm(request, request.POST)

    if form.is_valid():
        form.save()
    else:
        result["error"] = True

    return JsonResponse(result)



from __future__ import unicode_literals

import json
import os

from django import template

try:
    from django.core.urlresolvers import reverse
except ImportError:  # Django 1.11
    from django.urls import reverse

from django.contrib.admin.widgets import RelatedFieldWidgetWrapper
from django.forms import (
    CheckboxInput,
    ModelChoiceField,
    ModelMultipleChoiceField,
    Select,
    SelectMultiple,
)
from django.utils.encoding import smart_text
from django.utils.formats import get_format
from django.utils.safestring import mark_safe

from jet import VERSION, settings
from jet.models import Bookmark
from jet.utils import (
    get_admin_site,
    get_menu_items,
    get_model_instance_label,
    get_model_queryset,
    get_possible_language_codes,
)

try:
    from urllib.parse import parse_qsl
except ImportError:
    from urlparse import parse_qsl


register = template.Library()
assignment_tag = (
    register.assignment_tag
    if hasattr(register, "assignment_tag")
    else register.simple_tag
)


@assignment_tag
def jet_get_date_format():
    return get_format("DATE_INPUT_FORMATS")[0]


@assignment_tag
def jet_get_time_format():
    return get_format("TIME_INPUT_FORMATS")[0]


@assignment_tag
def jet_get_datetime_format():
    return get_format("DATETIME_INPUT_FORMATS")[0]


@assignment_tag(takes_context=True)
def jet_get_menu(context):
    return get_menu_items(context)


@assignment_tag
def jet_get_bookmarks(user):
    if user is None:
        return None
    return Bookmark.objects.filter(user=user.pk)


@register.filter
def jet_is_checkbox(field):
    return field.field.widget.__class__.__name__ == CheckboxInput().__class__.__name__


@register.filter
def jet_select2_lookups(field):
    if hasattr(field, "field") and (
        isinstance(field.field, ModelChoiceField)
        or isinstance(field.field, ModelMultipleChoiceField)
    ):
        qs = field.field.queryset
        model = qs.model

        if getattr(model, "autocomplete_search_fields", None) and getattr(
            field.field, "autocomplete", True
        ):
            choices = []
            app_label = model._meta.app_label
            model_name = model._meta.object_name

            attrs = {
                "class": "ajax",
                "data-app-label": app_label,
                "data-model": model_name,
                "data-ajax--url": reverse("jet:model_lookup"),
            }

            initial_value = field.value()

            if hasattr(field, "field") and isinstance(
                field.field, ModelMultipleChoiceField
            ):
                if initial_value:
                    initial_objects = model.objects.filter(pk__in=initial_value)
                    choices.extend(
                        [
                            (
                                initial_object.pk,
                                get_model_instance_label(initial_object),
                            )
                            for initial_object in initial_objects
                        ]
                    )

                if isinstance(field.field.widget, RelatedFieldWidgetWrapper):
                    field.field.widget.widget = SelectMultiple(attrs)
                else:
                    field.field.widget = SelectMultiple(attrs)
                field.field.choices = choices
            elif hasattr(field, "field") and isinstance(field.field, ModelChoiceField):
                if initial_value:
                    try:
                        initial_object = model.objects.get(pk=initial_value)
                        attrs["data-object-id"] = initial_value
                        choices.append(
                            (
                                initial_object.pk,
                                get_model_instance_label(initial_object),
                            )
                        )
                    except model.DoesNotExist:
                        pass

                if isinstance(field.field.widget, RelatedFieldWidgetWrapper):
                    field.field.widget.widget = Select(attrs)
                else:
                    field.field.widget = Select(attrs)
                field.field.choices = choices

    return field


@assignment_tag(takes_context=True)
def jet_get_current_theme(context):
    if "request" in context and "JET_THEME" in context["request"].COOKIES:
        theme = context["request"].COOKIES["JET_THEME"]
        if isinstance(settings.JET_THEMES, list) and len(settings.JET_THEMES) > 0:
            for conf_theme in settings.JET_THEMES:
                if isinstance(conf_theme, dict) and conf_theme.get("theme") == theme:
                    return theme
    return settings.JET_DEFAULT_THEME


@assignment_tag
def jet_get_themes():
    return settings.JET_THEMES


@assignment_tag
def jet_get_current_version():
    return VERSION


@register.filter
def jet_append_version(url):
    if "?" in url:
        return "%s&v=%s" % (url, VERSION)
    else:
        return "%s?v=%s" % (url, VERSION)


@assignment_tag
def jet_get_side_menu_compact():
    return settings.JET_SIDE_MENU_COMPACT


@assignment_tag
def jet_change_form_sibling_links_enabled():
    return settings.JET_CHANGE_FORM_SIBLING_LINKS


def jet_sibling_object(context, next):
    original = context.get("original")

    if not original:
        return

    model = type(original)
    preserved_filters_plain = context.get("preserved_filters", "")
    preserved_filters = dict(parse_qsl(preserved_filters_plain))
    admin_site = get_admin_site(context)

    if admin_site is None:
        return

    request = context.get("request")
    queryset = get_model_queryset(
        admin_site, model, request, preserved_filters=preserved_filters
    )

    if queryset is None:
        return

    sibling_object = None
    object_pks = list(queryset.values_list("pk", flat=True))

    try:
        index = object_pks.index(original.pk)
        sibling_index = index + 1 if next else index - 1
        exists = sibling_index < len(object_pks) if next else sibling_index >= 0
        sibling_object = queryset.get(pk=object_pks[sibling_index]) if exists else None
    except ValueError:
        pass

    if sibling_object is None:
        return

    url = reverse(
        "%s:%s_%s_change"
        % (admin_site.name, model._meta.app_label, model._meta.model_name),
        args=(sibling_object.pk,),
    )

    if preserved_filters_plain != "":
        url += "?" + preserved_filters_plain

    return {"label": str(sibling_object), "url": url}


@assignment_tag(takes_context=True)
def jet_previous_object(context):
    return jet_sibling_object(context, False)


@assignment_tag(takes_context=True)
def jet_next_object(context):
    return jet_sibling_object(context, True)


@assignment_tag(takes_context=True)
def jet_popup_response_data(context):
    if context.get("popup_response_data"):
        return context["popup_response_data"]

    return json.dumps(
        {
            "action": context.get("action"),
            "value": context.get("value") or context.get("pk_value"),
            "obj": smart_text(context.get("obj")),
            "new_value": context.get("new_value"),
        }
    )


@assignment_tag(takes_context=True)
def jet_delete_confirmation_context(context):
    if (
        context.get("deletable_objects") is None
        and context.get("deleted_objects") is None
    ):
        return ""
    return mark_safe('<div class="delete-confirmation-marker"></div>')


@assignment_tag
def jet_static_translation_urls():
    language_codes = get_possible_language_codes()

    urls = []
    url_templates = [
        "jet/js/i18n/jquery-ui/datepicker-__LANGUAGE_CODE__.js",
        "jet/js/i18n/jquery-ui-timepicker/jquery.ui.timepicker-__LANGUAGE_CODE__.js",
        "jet/js/i18n/select2/__LANGUAGE_CODE__.js",
    ]

    static_dir = os.path.join(os.path.dirname(os.path.dirname(__file__)), "static")

    for tpl in url_templates:
        for language_code in language_codes:
            url = tpl.replace("__LANGUAGE_CODE__", language_code)
            path = os.path.join(static_dir, url)

            if os.path.exists(path):
                urls.append(url)
                break

    return urls



try:
    from django.core.urlresolvers import reverse
except ImportError:  # Django 1.11
    from django.urls import reverse

from django.conf.urls import url
from django.contrib import messages
from django.http import HttpResponse
from django.shortcuts import redirect
from django.utils.translation import ugettext_lazy as _

from httplib2 import ServerNotFoundError
from oauth2client.client import FlowExchangeError

from jet.dashboard import dashboard
from jet.dashboard.dashboard_modules.google_analytics import (
    GoogleAnalyticsClient,
    ModuleCredentialStorage,
)
from jet.dashboard.models import UserDashboardModule


def google_analytics_grant_view(request, pk):
    redirect_uri = request.build_absolute_uri(
        reverse("jet-dashboard:google-analytics-callback")
    )
    client = GoogleAnalyticsClient(redirect_uri=redirect_uri)
    return redirect(client.get_oauth_authorize_url(pk))


def google_analytics_revoke_view(request, pk):
    try:
        module = UserDashboardModule.objects.get(pk=pk)
        ModuleCredentialStorage(module).delete()
        return redirect(
            reverse("jet-dashboard:update_module", kwargs={"pk": module.pk})
        )
    except UserDashboardModule.DoesNotExist:
        return HttpResponse(_("Module not found"))


def google_analytics_callback_view(request):
    module = None

    try:
        state = request.GET["state"]
        module = UserDashboardModule.objects.get(pk=state)

        redirect_uri = request.build_absolute_uri(
            reverse("jet-dashboard:google-analytics-callback")
        )
        client = GoogleAnalyticsClient(redirect_uri=redirect_uri)
        client.set_credential_from_request(request)

        ModuleCredentialStorage(module).put(client.credential)
    except (FlowExchangeError, ValueError, ServerNotFoundError):
        messages.error(request, _("API request failed."))
    except KeyError:
        return HttpResponse(_("Bad arguments"))
    except UserDashboardModule.DoesNotExist:
        return HttpResponse(_("Module not found"))

    return redirect(reverse("jet-dashboard:update_module", kwargs={"pk": module.pk}))


dashboard.urls.register_urls(
    [
        url(
            r"^google-analytics/grant/(?P<pk>\d+)/$",
            google_analytics_grant_view,
            name="google-analytics-grant",
        ),
        url(
            r"^google-analytics/revoke/(?P<pk>\d+)/$",
            google_analytics_revoke_view,
            name="google-analytics-revoke",
        ),
        url(
            r"^google-analytics/callback/",
            google_analytics_callback_view,
            name="google-analytics-callback",
        ),
    ]
)



# encoding: utf-8
import datetime
import json

from django import forms

try:
    from django.core.urlresolvers import reverse
except ImportError:  # Django 1.11
    from django.urls import reverse

from django.conf import settings
from django.forms import Widget
from django.utils import formats
from django.utils.encoding import force_text
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.text import capfirst
from django.utils.translation import ugettext_lazy as _

from googleapiclient.discovery import build
import httplib2
from oauth2client.client import (
    AccessTokenRefreshError,
    OAuth2Credentials,
    Storage,
    flow_from_clientsecrets,
)

from jet.dashboard.modules import DashboardModule

try:
    from django.utils.encoding import force_unicode
except ImportError:
    from django.utils.encoding import force_text as force_unicode

try:
    from django.forms.utils import flatatt
except ImportError:
    from django.forms.util import flatatt

JET_MODULE_GOOGLE_ANALYTICS_CLIENT_SECRETS_FILE = getattr(
    settings, "JET_MODULE_GOOGLE_ANALYTICS_CLIENT_SECRETS_FILE", ""
)


class ModuleCredentialStorage(Storage):
    def __init__(self, module):
        super(ModuleCredentialStorage, self).__init__()
        self.module = module

    def locked_get(self):
        pass

    def locked_put(self, credentials):
        pass

    def locked_delete(self):
        pass

    def get(self):
        try:
            settings = json.loads(self.module.settings)
            credential = settings["credential"]
            return OAuth2Credentials.from_json(credential)
        except (ValueError, KeyError):
            return None

    def put(self, credentials):
        self.module.update_settings({"credential": credentials.to_json()})

    def delete(self):
        self.module.pop_settings(("credential",))


class GoogleAnalyticsClient:
    credential = None
    analytics_service = None

    def __init__(self, storage=None, redirect_uri=None):
        self.FLOW = flow_from_clientsecrets(
            JET_MODULE_GOOGLE_ANALYTICS_CLIENT_SECRETS_FILE,
            scope="https://www.googleapis.com/auth/analytics.readonly",
            redirect_uri=redirect_uri,
            prompt="consent",
        )

        if storage is not None:
            credential = storage.get()
            credential.set_store(storage)
            self.set_credential(credential)

    def get_oauth_authorize_url(self, state=""):
        self.FLOW.params["state"] = state
        authorize_url = self.FLOW.step1_get_authorize_url()
        return authorize_url

    def set_credential(self, credential):
        self.credential = credential
        self.set_analytics_service(self.credential)

    def set_credential_from_request(self, request):
        self.set_credential(self.FLOW.step2_exchange(request.GET))

    def set_analytics_service(self, credential):
        http = httplib2.Http()
        http = credential.authorize(http)
        self.analytics_service = build("analytics", "v3", http=http)

    def api_profiles(self):
        if self.analytics_service is None:
            return None, None

        try:
            profiles = (
                self.analytics_service.management()
                .profiles()
                .list(accountId="~all", webPropertyId="~all")
                .execute()
            )

            return profiles["items"], None
        except (TypeError, KeyError) as e:
            return None, e

    def api_ga(self, profile_id, date1, date2, group=None):
        if self.analytics_service is None:
            return None, None

        if group == "day":
            dimensions = "ga:date"
        elif group == "week":
            dimensions = "ga:year,ga:week"
        elif group == "month":
            dimensions = "ga:year,ga:month"
        else:
            dimensions = ""

        try:
            data = (
                self.analytics_service.data()
                .ga()
                .get(
                    ids="ga:" + profile_id,
                    start_date=date1.strftime("%Y-%m-%d"),
                    end_date=date2.strftime("%Y-%m-%d"),
                    metrics="ga:users,ga:sessions,ga:pageviews",
                    dimensions=dimensions,
                )
                .execute()
            )

            return data, None
        except TypeError as e:
            return None, e


class CredentialWidget(Widget):
    module = None

    def render(self, name, value, attrs=None):
        if value and len(value) > 0:
            link = '<a href="%s">%s</a>' % (
                reverse(
                    "jet-dashboard:google-analytics-revoke",
                    kwargs={"pk": self.module.model.pk},
                ),
                force_text(_("Revoke access")),
            )
        else:
            link = '<a href="%s">%s</a>' % (
                reverse(
                    "jet-dashboard:google-analytics-grant",
                    kwargs={"pk": self.module.model.pk},
                ),
                force_text(_("Grant access")),
            )

        attrs = self.build_attrs(
            {
                "type": "hidden",
                "name": "credential",
            }
        )
        attrs["value"] = force_unicode(value) if value else ""

        return format_html("%s<input{} />" % link, flatatt(attrs))


class GoogleAnalyticsSettingsForm(forms.Form):
    credential = forms.CharField(label=_("Access"), widget=CredentialWidget)
    counter = forms.ChoiceField(label=_("Counter"))
    period = forms.ChoiceField(
        label=_("Statistics period"),
        choices=(
            (0, _("Today")),
            (6, _("Last week")),
            (30, _("Last month")),
            (31 * 3 - 1, _("Last quarter")),
            (364, _("Last year")),
        ),
    )

    def set_module(self, module):
        self.fields["credential"].widget.module = module
        self.set_counter_choices(module)

    def set_counter_choices(self, module):
        counters = module.counters()
        if counters is not None:
            self.fields["counter"].choices = (("", "-- %s --" % force_text(_("none"))),)
            self.fields["counter"].choices.extend(
                map(lambda x: (x["id"], x["websiteUrl"]), counters)
            )
        else:
            label = (
                force_text(_("grant access first"))
                if module.credential is None
                else force_text(_("counters loading failed"))
            )
            self.fields["counter"].choices = (("", "-- %s -- " % label),)


class GoogleAnalyticsChartSettingsForm(GoogleAnalyticsSettingsForm):
    show = forms.ChoiceField(
        label=_("Show"),
        choices=(
            ("ga:users", capfirst(_("users"))),
            ("ga:sessions", capfirst(_("sessions"))),
            ("ga:pageviews", capfirst(_("views"))),
        ),
    )
    group = forms.ChoiceField(
        label=_("Group"),
        choices=(
            ("day", _("By day")),
            ("week", _("By week")),
            ("month", _("By month")),
        ),
    )


class GoogleAnalyticsPeriodVisitorsSettingsForm(GoogleAnalyticsSettingsForm):
    group = forms.ChoiceField(
        label=_("Group"),
        choices=(
            ("day", _("By day")),
            ("week", _("By week")),
            ("month", _("By month")),
        ),
    )


class GoogleAnalyticsBase(DashboardModule):
    settings_form = GoogleAnalyticsSettingsForm
    ajax_load = True
    contrast = True
    period = None
    credential = None
    counter = None
    error = None
    storage = None

    def __init__(self, title=None, period=None, **kwargs):
        kwargs.update({"period": period})
        super(GoogleAnalyticsBase, self).__init__(title, **kwargs)

    def settings_dict(self):
        return {
            "period": self.period,
            "credential": self.credential,
            "counter": self.counter,
        }

    def load_settings(self, settings):
        try:
            self.period = int(settings.get("period"))
        except TypeError:
            self.period = 0
        self.credential = settings.get("credential")
        self.storage = ModuleCredentialStorage(self.model)
        self.counter = settings.get("counter")

    def init_with_context(self, context):
        raise NotImplementedError(
            "subclasses of GoogleAnalytics must provide a init_with_context() method"
        )

    def counters(self):
        try:
            client = GoogleAnalyticsClient(self.storage)
            profiles, exception = client.api_profiles()
            return profiles
        except Exception:
            return None

    def get_grouped_date(self, data, group):
        if group == "week":
            date = datetime.datetime.strptime(
                "%s-%s-%s" % (data["ga_year"], data["ga_week"], "0"), "%Y-%W-%w"
            )
        elif group == "month":
            date = datetime.datetime.strptime(
                data["ga_year"] + data["ga_month"], "%Y%m"
            )
        else:
            date = datetime.datetime.strptime(data["ga_date"], "%Y%m%d")
        return date

    def format_grouped_date(self, data, group):
        date = self.get_grouped_date(data, group)

        if group == "week":
            date = u"%s — %s" % (
                (date - datetime.timedelta(days=6)).strftime("%d.%m"),
                date.strftime("%d.%m"),
            )
        elif group == "month":
            date = date.strftime("%b, %Y")
        else:
            date = formats.date_format(date, "DATE_FORMAT")
        return date

    def counter_attached(self):
        if self.credential is None:
            self.error = mark_safe(
                _(
                    'Please <a href="%s">attach Google account and choose Google Analytics counter</a> to start using widget'
                )
                % reverse("jet-dashboard:update_module", kwargs={"pk": self.model.pk})
            )
            return False
        elif self.counter is None:
            self.error = mark_safe(
                _(
                    'Please <a href="%s">select Google Analytics counter</a> to start using widget'
                )
                % reverse("jet-dashboard:update_module", kwargs={"pk": self.model.pk})
            )
            return False
        else:
            return True

    def api_ga(self, group=None):
        if self.counter_attached():
            date1 = datetime.datetime.now() - datetime.timedelta(days=self.period)
            date2 = datetime.datetime.now()

            try:
                client = GoogleAnalyticsClient(self.storage)
                result, exception = client.api_ga(self.counter, date1, date2, group)

                if exception is not None:
                    raise exception

                return result
            except Exception as e:
                error = _("API request failed.")
                if isinstance(e, AccessTokenRefreshError):
                    error += _(
                        ' Try to <a href="%s">revoke and grant access</a> again'
                    ) % reverse(
                        "jet-dashboard:update_module", kwargs={"pk": self.model.pk}
                    )
                self.error = mark_safe(error)


class GoogleAnalyticsVisitorsTotals(GoogleAnalyticsBase):
    """
    Google Analytics widget that shows total number of users, sessions and viewers for a particular period of time.
    Period may be following: Today, Last week, Last month, Last quarter, Last year
    """

    title = _("Google Analytics visitors totals")
    template = "jet.dashboard/modules/google_analytics_visitors_totals.html"

    #: Which period should be displayed. Allowed values - integer of days
    period = None

    def __init__(self, title=None, period=None, **kwargs):
        kwargs.update({"period": period})
        super(GoogleAnalyticsVisitorsTotals, self).__init__(title, **kwargs)

    def init_with_context(self, context):
        result = self.api_ga()

        if result is not None:
            try:
                self.children.append(
                    {
                        "title": _("users"),
                        "value": result["totalsForAllResults"]["ga:users"],
                    }
                )
                self.children.append(
                    {
                        "title": _("sessions"),
                        "value": result["totalsForAllResults"]["ga:sessions"],
                    }
                )
                self.children.append(
                    {
                        "title": _("views"),
                        "value": result["totalsForAllResults"]["ga:pageviews"],
                    }
                )
            except KeyError:
                self.error = _("Bad server response")


class GoogleAnalyticsVisitorsChart(GoogleAnalyticsBase):
    """
    Google Analytics widget that shows users/sessions/viewer chart for a particular period of time.
    Data is grouped by day, week or month
    Period may be following: Today, Last week, Last month, Last quarter, Last year
    """

    title = _("Google Analytics visitors chart")
    template = "jet.dashboard/modules/google_analytics_visitors_chart.html"
    style = "overflow-x: auto;"

    #: Which period should be displayed. Allowed values - integer of days
    period = None

    #: What data should be shown. Possible values: ``ga:users``, ``ga:sessions``, ``ga:pageviews``
    show = None

    #: Sets grouping of data. Possible values: ``day``, ``week``, ``month``
    group = None
    settings_form = GoogleAnalyticsChartSettingsForm

    class Media:
        js = (
            "jet.dashboard/vendor/chart.js/Chart.min.js",
            "jet.dashboard/dashboard_modules/google_analytics.js",
        )

    def __init__(self, title=None, period=None, show=None, group=None, **kwargs):
        kwargs.update({"period": period, "show": show, "group": group})
        super(GoogleAnalyticsVisitorsChart, self).__init__(title, **kwargs)

    def settings_dict(self):
        settings = super(GoogleAnalyticsVisitorsChart, self).settings_dict()
        settings["show"] = self.show
        settings["group"] = self.group
        return settings

    def load_settings(self, settings):
        super(GoogleAnalyticsVisitorsChart, self).load_settings(settings)
        self.show = settings.get("show")
        self.group = settings.get("group")

    def init_with_context(self, context):
        result = self.api_ga(self.group)

        if result is not None:
            try:
                for data in result["rows"]:
                    row_data = {}

                    i = 0
                    for column in result["columnHeaders"]:
                        row_data[column["name"].replace(":", "_")] = data[i]
                        i += 1

                    date = self.get_grouped_date(row_data, self.group)
                    self.children.append((date, row_data[self.show.replace(":", "_")]))
            except KeyError:
                self.error = _("Bad server response")


class GoogleAnalyticsPeriodVisitors(GoogleAnalyticsBase):
    """
    Google Analytics widget that shows users, sessions and viewers for a particular period of time.
    Data is grouped by day, week or month
    Period may be following: Today, Last week, Last month, Last quarter, Last year
    """

    title = _("Google Analytics period visitors")
    template = "jet.dashboard/modules/google_analytics_period_visitors.html"

    #: Which period should be displayed. Allowed values - integer of days
    period = None

    #: Sets grouping of data. Possible values: ``day``, ``week``, ``month``
    group = None
    contrast = False
    settings_form = GoogleAnalyticsPeriodVisitorsSettingsForm

    def __init__(self, title=None, period=None, group=None, **kwargs):
        kwargs.update({"period": period, "group": group})
        super(GoogleAnalyticsPeriodVisitors, self).__init__(title, **kwargs)

    def settings_dict(self):
        settings = super(GoogleAnalyticsPeriodVisitors, self).settings_dict()
        settings["group"] = self.group
        return settings

    def load_settings(self, settings):
        super(GoogleAnalyticsPeriodVisitors, self).load_settings(settings)
        self.group = settings.get("group")

    def init_with_context(self, context):
        result = self.api_ga(self.group)

        if result is not None:
            try:
                for data in reversed(result["rows"]):
                    row_data = {}

                    i = 0
                    for column in result["columnHeaders"]:
                        row_data[column["name"].replace(":", "_")] = data[i]
                        i += 1

                    date = self.format_grouped_date(row_data, self.group)
                    self.children.append((date, row_data))
            except KeyError:
                self.error = _("Bad server response")



from django.contrib.admin import RelatedFieldListFilter
from django.utils.encoding import smart_text
from django.utils.html import format_html

try:
    from django.core.urlresolvers import reverse
except ImportError:  # Django 1.11
    from django.urls import reverse

try:
    from django.contrib.admin.utils import get_model_from_relation
except ImportError:  # Django 1.6
    from django.contrib.admin.util import get_model_from_relation

try:
    from django.forms.utils import flatatt
except ImportError:  # Django 1.6
    from django.forms.util import flatatt


class RelatedFieldAjaxListFilter(RelatedFieldListFilter):
    template = "jet/related_field_ajax_list_filter.html"
    ajax_attrs = None

    def has_output(self):
        return True

    def field_choices(self, field, request, model_admin):
        model = (
            field.remote_field.model
            if hasattr(field, "remote_field")
            else field.related_field.model
        )
        app_label = model._meta.app_label
        model_name = model._meta.object_name

        self.ajax_attrs = format_html(
            "{0}",
            flatatt(
                {
                    "data-app-label": app_label,
                    "data-model": model_name,
                    "data-ajax--url": reverse("jet:model_lookup"),
                    "data-queryset--lookup": self.lookup_kwarg,
                }
            ),
        )

        if self.lookup_val is None:
            return []

        other_model = get_model_from_relation(field)
        if hasattr(field, "rel"):
            rel_name = field.rel.get_related_field().name
        else:
            rel_name = other_model._meta.pk.name

        queryset = model._default_manager.filter(**{rel_name: self.lookup_val}).all()
        return [(x._get_pk_val(), smart_text(x)) for x in queryset]


try:
    from collections import OrderedDict

    from django import forms
    from django.contrib.admin.widgets import AdminDateWidget
    from django.utils.translation import ugettext as _

    from rangefilter.filter import DateRangeFilter as OriginalDateRangeFilter

    class DateRangeFilter(OriginalDateRangeFilter):
        def get_template(self):
            return "rangefilter/date_filter.html"

        def _get_form_fields(self):
            # this is here, because in parent DateRangeFilter AdminDateWidget
            # could be imported from django-suit
            return OrderedDict(
                (
                    (
                        self.lookup_kwarg_gte,
                        forms.DateField(
                            label="",
                            widget=AdminDateWidget(
                                attrs={"placeholder": _("From date")}
                            ),
                            localize=True,
                            required=False,
                        ),
                    ),
                    (
                        self.lookup_kwarg_lte,
                        forms.DateField(
                            label="",
                            widget=AdminDateWidget(attrs={"placeholder": _("To date")}),
                            localize=True,
                            required=False,
                        ),
                    ),
                )
            )

        @staticmethod
        def _get_media():
            css = [
                "style.css",
            ]
            return forms.Media(
                css={"all": ["range_filter/css/%s" % path for path in css]}
            )


except ImportError:
    pass



from django.conf import settings

# Theme
JET_DEFAULT_THEME = getattr(settings, "JET_DEFAULT_THEME", "default")
JET_THEMES = getattr(settings, "JET_THEMES", [])

# Side menu
JET_SIDE_MENU_COMPACT = getattr(settings, "JET_SIDE_MENU_COMPACT", False)
JET_SIDE_MENU_ITEMS = getattr(settings, "JET_SIDE_MENU_ITEMS", None)
JET_SIDE_MENU_CUSTOM_APPS = getattr(settings, "JET_SIDE_MENU_CUSTOM_APPS", None)

# Improved usability
JET_CHANGE_FORM_SIBLING_LINKS = getattr(settings, "JET_CHANGE_FORM_SIBLING_LINKS", True)
